<?php

namespace Teknisa\FIS\Factories;

use Doctrine\ORM\EntityManager;

class Fis25000ParametrizacaoContribuicoesFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function deleteContaContabilOutrasContribuicoes ($nrorg, $cdempresa, $cdoutreccont){
        $sql = "DELETE
                FROM OUTRECCONTCTBL
                WHERE NRORG = :NRORG AND
                CDEMPRESA    = :CDEMPRESA AND
                CDOUTRECCONT = :CDOUTRECCONT";

        $params = array(
            'NRORG'=>$nrorg,
            'CDEMPRESA'=>$cdempresa,
            'CDOUTRECCONT'=>$cdoutreccont,
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function deleteCodigoNatureza($nrorg,$cdtabelasped){
        $sql = "DELETE
                FROM CODNATURECSPEDCONTR
                WHERE CDTABELASPED = :P_CDTABELASPED
                AND NRORG = :P_NRORG";

        $params = array(
            'P_CDTABELASPED'=>$cdtabelasped,
            'P_NRORG'=>$nrorg
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function deleteClassificacaoFiscal($nrorg,$cdtabelasped){
        $sql = "DELETE
                FROM CODNCMNATUREC
                WHERE CDTABELASPED = :P_CDTABELASPED
                AND NRORG = :P_NRORG";

        $params = array(
            'P_CDTABELASPED'=>$cdtabelasped,
            'P_NRORG'=>$nrorg
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function verificaClassificaoFiscal($cdnaturec, $cdclasfisc, $cdtabelasped,$nrorg) {
        #sets parameter data
        $params                   = array();
        $params['P_CDNATUREC']    = $cdnaturec;
        $params['P_CDCLASFISC']   = $cdclasfisc;
        $params['P_CDTABELASPED'] = $cdtabelasped;
        $params['NRORG']          = $nrorg;
        #defines function query
        $sql = "SELECT CDCLASFISC
                           FROM CODNCMNATUREC
                          WHERE CDNATUREC = :P_CDNATUREC
                            AND CDCLASFISC = :P_CDCLASFISC
                            AND CDTABELASPED = :P_CDTABELASPED
                            AND NRORG = :NRORG";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function verificaNatureza($cdnaturec, $dtinivalid, $dtfinvalid, $cdtabelasped,$nrorg) {
        #sets parameter data
        $params                   = array();
        $params['P_CDNATUREC']    = $cdnaturec;
        $params['P_DTINIVALID']   = $dtinivalid;
        $params['P_DTFINVALID']   = $dtfinvalid;
        $params['P_CDTABELASPED'] = $cdtabelasped;
        $params['NRORG']          = $nrorg;
        #defines function query
        $sql = "SELECT CDNATUREC
                    FROM CODNATURECSPEDCONTR
                    WHERE CDNATUREC = :P_CDNATUREC
                        AND DTINIVALID = :P_DTINIVALID
                        AND (DTFINVALID = :P_DTFINVALID OR DTFINVALID IS NULL AND :P_DTFINVALID IS NULL)
                        AND CDTABELASPED = :P_CDTABELASPED
                        AND NRORG = :NRORG";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function buscaClassificaoFiscal($cdclasfisc, $nrorg) {
        #sets parameter data
        $params                 = array();
        $params['NRORG']        = $nrorg;
        $params['P_CDCLASFISC'] = $cdclasfisc;
        #defines function query
        $sql = "SELECT CDCLASFISC
                    FROM CLASFISCAL
                   WHERE CDCLASFISC LIKE REPLACE(REPLACE(:P_CDCLASFISC,'|',''),';','')||'%'
                   AND NRORG = :NRORG";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function validaIntervaloDatas($cdsitutribpico, $cdempresa, $cdnaturec, $dtinivalid, $vraliqcofins, $idoperpico, $cdoutreccont, $dtfimvalid, $vraliqpis ) {

        #sets parameter data
        $params                     = array();
        $params['P_CDSITUTRIBPICO'] = $cdsitutribpico;
        $params['P_CDEMPRESA']      = $cdempresa;
        $params['P_CDNATUREC']      = $cdnaturec;
        $params['P_DTINIVALID']     = $dtinivalid;
        $params['P_VRALIQCOFINS']   = $vraliqcofins;
        $params['P_IDOPERPICO']     = $idoperpico;
        $params['P_CDOUTRECCONT']   = $cdoutreccont;
        $params['P_DTFIMVALID']     = $dtfimvalid;
        $params['P_VRALIQPIS']      = $vraliqpis;

        #defines function query
        $sql = "SELECT CDEMPRESA,
                         CDSITUTRIBPICO,
                         IDOPERPICO,
                         VRALIQPIS,
                         VRALIQCOFINS,
                         CDNATUREC,
                         MAX(ERRO) AS ERRO
                FROM
                    (SELECT CDEMPRESA,
                         CDOUTRECCONT,
                         CDSITUTRIBPICO,
                         IDOPERPICO,
                         VRALIQPIS,
                         VRALIQCOFINS,
                         CDNATUREC,
                         DTINIVALID,
                         DTFIMVALID,

                        CASE
                        WHEN :P_DTFIMVALID IS NULL THEN
                        'DtFim_Inf'
                        WHEN :P_DTFIMVALID >= DTINIVALID THEN
                        'DtInt_Inv'
                        ELSE ''
                        END AS ERRO
                    FROM OUTRECCONT
                    WHERE CDEMPRESA = :P_CDEMPRESA
                            AND CDOUTRECCONT <> :P_CDOUTRECCONT
                            AND CDSITUTRIBPICO = :P_CDSITUTRIBPICO
                            AND IDOPERPICO = :P_IDOPERPICO
                            AND VRALIQPIS = :P_VRALIQPIS
                            AND VRALIQCOFINS = :P_VRALIQCOFINS
                            AND ((CDNATUREC = :P_CDNATUREC)
                            OR (CDNATUREC IS NULL
                            AND :P_CDNATUREC IS NULL))
                            AND :P_DTINIVALID < DTINIVALID
                    UNION ALL
                        SELECT CDEMPRESA,
                         CDOUTRECCONT,
                         CDSITUTRIBPICO,
                         IDOPERPICO,
                         VRALIQPIS,
                         VRALIQCOFINS,
                         CDNATUREC,
                         DTINIVALID,
                         DTFIMVALID,

                        CASE
                        WHEN DTFIMVALID IS NULL THEN
                        'DtFimOld_Inf'
                        WHEN DTFIMVALID >= :P_DTINIVALID THEN
                        'DtInt_Inv'
                        ELSE ''
                        END AS ERRO
                    FROM OUTRECCONT
                    WHERE CDEMPRESA = :P_CDEMPRESA
                            AND CDOUTRECCONT <> :P_CDOUTRECCONT
                            AND CDSITUTRIBPICO = :P_CDSITUTRIBPICO
                            AND IDOPERPICO = :P_IDOPERPICO
                            AND VRALIQPIS = :P_VRALIQPIS
                            AND VRALIQCOFINS = :P_VRALIQCOFINS
                            AND ((CDNATUREC = :P_CDNATUREC)
                            OR (CDNATUREC IS NULL
                            AND :P_CDNATUREC IS NULL))
                            AND :P_DTINIVALID > DTINIVALID
                    UNION ALL
                    SELECT CDEMPRESA,
                         CDOUTRECCONT,
                         CDSITUTRIBPICO,
                         IDOPERPICO,
                         VRALIQPIS,
                         VRALIQCOFINS,
                         CDNATUREC,
                         DTINIVALID,
                         DTFIMVALID,
                         'DtInt_Inv' AS ERRO
                    FROM OUTRECCONT
                    WHERE CDEMPRESA = :P_CDEMPRESA
                            AND CDOUTRECCONT <> :P_CDOUTRECCONT
                            AND CDSITUTRIBPICO = :P_CDSITUTRIBPICO
                            AND IDOPERPICO = :P_IDOPERPICO
                            AND VRALIQPIS = :P_VRALIQPIS
                            AND VRALIQCOFINS = :P_VRALIQCOFINS
                            AND ((CDNATUREC = :P_CDNATUREC)
                            OR (CDNATUREC IS NULL
                            AND :P_CDNATUREC IS NULL))
                            AND DTINIVALID = :P_DTINIVALID)
                GROUP BY  CDEMPRESA, CDSITUTRIBPICO, IDOPERPICO, VRALIQPIS, VRALIQCOFINS, CDNATUREC";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function validaContaContabil($cdempresa, $cdcontctbl, $cdnvcontctbl, $dtinivalid, $dtfimvalid) {

        #sets parameter data
        $params                   = array();
        $params['P_CDEMPRESA']    = $cdempresa;
        $params['P_CDCONTCTBL']   = $cdcontctbl;
        $params['P_CDNVCONTCTBL'] = $cdnvcontctbl;
        $params['P_DTINIVALID']   = $dtinivalid;
        $params['P_DTFIMVALID']   = $dtfimvalid;

        #defines function query
        $sql = "SELECT O.CDCONTCTBL,
                         CDNVCONTCTBL,
                         'S' AS IDERRO, CDSITUTRIBPICO, DTINIVALID, DTFIMVALID
                FROM OUTRECCONTCTBL O, CONTCTBL C, OUTRECCONT OU
                    WHERE O.CDCONTCTBL = C.CDCONTCTBL
                            AND OU.CDOUTRECCONT = O.CDOUTRECCONT
                            AND OU.CDEMPRESA = O.CDEMPRESA
                            AND O.CDEMPRESA = :P_CDEMPRESA
                            AND NOT ((OU.DTFIMVALID IS NULL
                            AND :P_DTINIVALID < OU.DTINIVALID
                            AND :P_DTFIMVALID < OU.DTINIVALID)
                            OR (:P_DTFIMVALID IS NULL
                            AND OU.DTINIVALID < :P_DTINIVALID
                            AND OU.DTFIMVALID < :P_DTINIVALID)
                            OR ((NOT OU.DTINIVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (:P_DTFIMVALID IS NOT NULL)
                            AND (NOT OU.DTFIMVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (OU.DTFIMVALID IS NOT NULL)))
                            AND TO_NUMBER(C.CDNVCONTCTBL) < TO_NUMBER(:P_CDNVCONTCTBL)
                            AND SUBSTR(:P_CDCONTCTBL,1,LENGTH(O.CDCONTCTBL)) = O.CDCONTCTBL
                UNION ALL
                SELECT CDCONTCTBL,
                         '' AS CDNVCONTCTBL, 'E' AS IDERRO, CDSITUTRIBPICO, DTINIVALID, DTFIMVALID
                FROM OUTRECCONTCTBL O, OUTRECCONT OU
                    WHERE O.CDCONTCTBL = :P_CDCONTCTBL
                            AND OU.CDOUTRECCONT = O.CDOUTRECCONT
                            AND OU.CDEMPRESA = O.CDEMPRESA
                            AND O.CDEMPRESA = :P_CDEMPRESA
                            AND NOT ((OU.DTFIMVALID IS NULL
                            AND :P_DTINIVALID < OU.DTINIVALID
                            AND :P_DTFIMVALID < OU.DTINIVALID)
                            OR (:P_DTFIMVALID IS NULL
                            AND OU.DTINIVALID < :P_DTINIVALID
                            AND OU.DTFIMVALID < :P_DTINIVALID)
                            OR ((NOT OU.DTINIVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (:P_DTFIMVALID IS NOT NULL)
                            AND (NOT OU.DTFIMVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (OU.DTFIMVALID IS NOT NULL)))
                UNION ALL
                SELECT O.CDCONTCTBL,
                         CDNVCONTCTBL,
                         'I' AS IDERRO, CDSITUTRIBPICO, DTINIVALID, DTFIMVALID
                FROM OUTRECCONTCTBL O, CONTCTBL C, OUTRECCONT OU
                    WHERE O.CDCONTCTBL = C.CDCONTCTBL
                            AND OU.CDOUTRECCONT = O.CDOUTRECCONT
                            AND OU.CDEMPRESA = O.CDEMPRESA
                            AND O.CDEMPRESA = :P_CDEMPRESA
                            AND TO_NUMBER(C.CDNVCONTCTBL) > TO_NUMBER(:P_CDNVCONTCTBL)
                            AND SUBSTR(O.CDCONTCTBL,1,LENGTH(:P_CDCONTCTBL)) = :P_CDCONTCTBL
                            AND NOT ((OU.DTFIMVALID IS NULL
                            AND :P_DTINIVALID < OU.DTINIVALID
                            AND :P_DTFIMVALID < OU.DTINIVALID)
                            OR (:P_DTFIMVALID IS NULL
                            AND OU.DTINIVALID < :P_DTINIVALID
                            AND OU.DTFIMVALID < :P_DTINIVALID)
                            OR ((NOT OU.DTINIVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (:P_DTFIMVALID IS NOT NULL)
                            AND (NOT OU.DTFIMVALID
                        BETWEEN :P_DTINIVALID
                            AND :P_DTFIMVALID)
                            AND (OU.DTFIMVALID IS NOT NULL)))";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function validaBenfeitorias($cdempresa, $cdcontctbl, $cdnvcontctbl, $nrseqbenfeitoria) {

        #sets parameter data
        $params                       = array();
        $params['P_CDEMPRESA']        = $cdempresa;
        $params['P_CDCONTCTBL']       = $cdcontctbl;
        $params['P_CDNVCONTCTBL']     = $cdnvcontctbl;
        $params['P_NRSEQBENFEITORIA'] = $nrseqbenfeitoria;

        #defines function query
        $sql = "SELECT B.CDCONTCTBL, C.CDNVCONTCTBL, 'S' AS IDERRO
                FROM BENFEITORIA B, CONTCTBL C
                WHERE B.CDEMPRESA = :P_CDEMPRESA
                        AND B.CDCONTCTBL = C.CDCONTCTBL
                        AND (B.NRSEQBENFEITORIA <> :P_NRSEQBENFEITORIA
                        OR :P_NRSEQBENFEITORIA IS NULL)
                        AND TO_NUMBER(C.CDNVCONTCTBL) < TO_NUMBER(:P_CDNVCONTCTBL)
                        AND SUBSTR(:P_CDCONTCTBL,1,LENGTH(B.CDCONTCTBL)) = B.CDCONTCTBL
                UNION
                ALL SELECT B.CDCONTCTBL,
                         '' AS CDNVCONTCTBL, 'E' AS IDERRO
                FROM BENFEITORIA B
                WHERE B.CDEMPRESA = :P_CDEMPRESA
                        AND B.CDCONTCTBL = :P_CDCONTCTBL
                        AND (B.NRSEQBENFEITORIA <> :P_NRSEQBENFEITORIA
                        OR :P_NRSEQBENFEITORIA IS NULL)
                UNION
                ALL SELECT B.CDCONTCTBL,
                         C.CDNVCONTCTBL,
                         'I' AS IDERRO
                FROM BENFEITORIA B, CONTCTBL C
                WHERE B.CDEMPRESA = :P_CDEMPRESA
                        AND B.CDCONTCTBL = C.CDCONTCTBL
                        AND (B.NRSEQBENFEITORIA <> :P_NRSEQBENFEITORIA
                        OR :P_NRSEQBENFEITORIA IS NULL)
                        AND TO_NUMBER(C.CDNVCONTCTBL) > TO_NUMBER(:P_CDNVCONTCTBL)
                        AND SUBSTR(B.CDCONTCTBL,1,LENGTH(:P_CDCONTCTBL)) = :P_CDCONTCTBL ";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }


    public function validaAliquota($cdsitutribnfe, $vraliquota, $cdimposto) {
        #sets parameter data
        $params                    = array();
        $params['P_CDIMPOSTO']     = $cdimposto;
        $params['P_CDSITUTRIBNFE'] = $cdsitutribnfe;
        $params['P_VRALIQUOTA']    = $vraliquota;
        #defines function query
        $sql = "SELECT VRALIQUOTA
                FROM ALIQUOTACSTNFE A
                WHERE   CDIMPOSTO = :P_CDIMPOSTO
                        AND CDSITUTRIBNFE = :P_CDSITUTRIBNFE
                        AND VRALIQUOTA    = :P_VRALIQUOTA";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function geraCodigOutrasContribuicoes($cdempresa ,$nrorg){
        #sets parameter data
        $params                = array();
        $params['P_CDEMPRESA'] = $cdempresa;
        $params['P_NRORG']     = $nrorg;
        #defines function query
        $sql = "SELECT  NVL(MAX(CDOUTRECCONT)+1,1) AS CDOUTRECCONT FROM  OUTRECCONT
                WHERE CDEMPRESA  = :P_CDEMPRESA
                      AND NRORG  = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
  

}