<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Acessorias\Services\Fis85000SpedService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;

class Fis85000SpedController {

    protected $spedService;

    public function __construct(EntityManager $entityManager, Fis85000SpedService $spedService) {
        $this->entityManager            = $entityManager;
        $this->spedService = $spedService;
        set_time_limit(0);
    }

    /*
     * Função para realizar a exportação SPED.
     */
    public function exportarSped(DTO\Request $request, DTO\Response $response) {
        try {
        
            $row = $request->getRow();
            
            $row["NMARQUIVO"] = str_replace(' ', '', $row["NMARQUIVO"]);

            // Chama a função de validação.
            $fileName = $this->spedService->exportarSped($row);

            // Retorna o nome do arquivo exportado.
            $response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function findAgendamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            
            $rows = $this->spedService->findAgendamento();
            
            $dataSet = new DataSet('agendobrigacess_fis85000', $rows);
            $response->setDataSets(array($dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function saveAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $fileName = $this->spedService->saveAgendamentoSped($row);
            
            $this->entityManager->commit();

            // Retorna o nome do arquivo exportado.
            $response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $fileName = $this->spedService->deleteAgendamento($row);
            
            $this->entityManager->commit();

            // Retorna o nome do arquivo exportado.
            $response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteArquivo(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach($request->getDataSet()->getRows() as $row){
                $this->spedService->deleteArquivos($row);
            }
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getFileServerUrl(DTO\Request $request, DTO\Response $response) {
        try {
            
            $fileName = $this->spedService->getFileServerUrl();
            
            $response->addDataset(new DataSet('sucess', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function execAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
                                    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getParameters();
            
            $_SESSION['_sf2_attributes']['NRORG'] = $row['NRORG'];
            $_SESSION['_sf2_attributes']['ORGANIZATION_ID'] = $row['NRORG'];
            
            $this->spedService->execAgendamento($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

}