<?php

namespace Teknisa\FIS\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Factories\Fis07200LivroMagneticoFactory;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;

class Fis07200LivroMagneticoService {
    protected $entityManager;
    protected $environment;
    protected $nrorg;
    protected $factoryLivroMagnetico;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Fis07200LivroMagneticoFactory $factoryLivroMagnetico) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->nrorg = $environment->getNrorgTrab();
        $this->operador = $environment->getCdOperador();
        $this->factoryLivroMagnetico = $factoryLivroMagnetico;
    }

    public function deletaLayoutCFOPLivroMag($cdlivromag){
        $this->factoryLivroMagnetico->deletaLayoutCFOPLivroMag(array(
            "CDLIVROMAG" => $cdlivromag,
            "NRORG" => $this->nrorg
        ));
    }
    public function deletaCadastroTipo($cdlivromag){
        $this->factoryLivroMagnetico->deletaCadastroTipo(array(
            "CDLIVROMAG" => $cdlivromag,
            "NRORG" => $this->nrorg
        ));
    }
    public function deletaCadastroLayout($cdlivromag){
        $this->factoryLivroMagnetico->deletaCadastroLayout(array(
            "CDLIVROMAG" => $cdlivromag,
            "NRORG" => $this->nrorg
        ));
    }
    public function replicaTipoDocumentoCFOP($cdlivromagorig, $cdlivromagdest){
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->factoryLivroMagnetico->replicaTipoDocumentoCFOP(array(
            "NRORG" => $this->nrorg,
            "CDLIVROMAGORIG" => $cdlivromagorig,
            "CDLIVROMAGDEST" => $cdlivromagdest,
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->nrorg,
            "CDOPERINCLUSAO" => $this->operador
        ));
    }
      public function replicaCadastroTipo($cdlivromagorig, $cdlivromagdest){
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->factoryLivroMagnetico->replicaCadastroTipo(array(
            "NRORG" => $this->nrorg,
            "CDLIVROMAGORIG" => $cdlivromagorig,
            "CDLIVROMAGDEST" => $cdlivromagdest,
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->nrorg,
            "CDOPERINCLUSAO" => $this->operador
        ));
    }

    public function deletaGeneroItemProduto($cdlivromag){
        $this->factoryLivroMagnetico->deletaGeneroItemProduto(array(
            "CDLIVROMAG" => $cdlivromag,
            "NRORG" => $this->nrorg
        ));
    }
    public function saveLayoutLivroMag($row, $cdlivromag){
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->factoryLivroMagnetico->saveLayoutLivroMag(array(
            "NRORG" => $this->nrorg,
            "CDLIVROMAG" => $cdlivromag,
            "CDLAYOUTLMAG" => $row['CDLAYOUTLMAG'],
            "NMLAYOUTLMAG" => $row['NMLAYOUTLMAG'],
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->nrorg,
            "CDOPERINCLUSAO" => $this->operador
        ));
    }

    public function saveCFOPLayoutLivroMag($row, $cdlivromag){
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->factoryLivroMagnetico->saveCFOPLayoutLivroMag(array(
            "NRORG" => $this->nrorg,
            "CDLIVROMAG" => $cdlivromag,
            "CDLAYOUTLMAG" => $row['CDLAYOUTLMAG'],
            "CDCFOPLMAG" => $row['CDCFOPLMAG'],
            "DSCFOPLMAG" => $row['DSCFOPLMAG'],
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->nrorg,
            "CDOPERINCLUSAO" => $this->operador
        ));
    }

    public function genitemprodutoIsNew($row) {
        $genitprod = $this->factoryLivroMagnetico->getGenItemProduto(array(
            "NRORG" => $this->nrorg,
            "CDPRODUTO" => $row['CDPRODUTO'],
            "CDLIVROMAG" => $row['CDLIVROMAG']
        ));
        return $genitprod === false;
    }

    public function cadastraGenTipoItemProdAutomatico($row, $cdlivromag){
        $produtos = $this->factoryLivroMagnetico->getIntervaloProdutos( $this->nrorg, $row['CDPRODINI'], $row['CDPRODFIN'] );
        foreach ($produtos as $produto){
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $params = array("CDPRODUTO" => $produto['CDPRODUTO'], "CDLIVROMAG" => $cdlivromag);
            
            $result = $this->factoryLivroMagnetico->getGeneroProd( $this->nrorg, $row['CDGENPROD'], $cdlivromag );
                if( !$result ) {
                    $result = $this->factoryLivroMagnetico->getGeneroProd( $this->nrorg, $row['CDGENPROD'], 'MOBILE' );
                    $this->factoryLivroMagnetico->insertGeneroProd( $row['CDGENPROD'], $result['DSGENPROD'], $cdlivromag, $this->nrorg, $this->operador );
                }
                
            if ($this->genitemprodutoIsNew($params)) {
                $this->factoryLivroMagnetico->insertRelacProdItGen(array(
                    "NRORG" => $this->nrorg,
                    "CDLIVROMAG" => $cdlivromag,
                    "CDPRODUTO" => $produto['CDPRODUTO'],
                    "CDGENPROD" => $row['CDGENPROD'],
                    "CDTPITEMPROD" => $row['CDTPITEMPROD'],
                    "DTINCLUSAO" => $currentTime,
                    "NRORGINCLUSAO" => $this->nrorg,
                    "CDOPERINCLUSAO" => $this->operador
                ));
            } else {
                $this->factoryLivroMagnetico->updateRelacProdItGen(array(
                    "NRORG" => $this->nrorg,
                    "CDLIVROMAG" => $cdlivromag,
                    "CDPRODUTO" => $produto['CDPRODUTO'],
                    "CDGENPROD" => $row['CDGENPROD'],
                    "CDTPITEMPROD" => $row['CDTPITEMPROD'],
                    "DTULTATU" => $currentTime,
                    "NRORGULTATU" => $this->nrorg,
                    "CDOPERULTATU" => $this->operador
                ));
            }
        }
    }
    
    public function deletarLivMag($row){
        
        $this->factoryLivroMagnetico->deletaLayoutCFOPLivroMag( array(
            "NRORG" => $row['NRORG'],
            "CDLIVROMAG" => $row['CDLIVROMAG'] ), true );
        
    }
    
    public function deletarCfopLayout($row){
        
        $this->factoryLivroMagnetico->deletaTipoDocumentoCfopLayout(array(
            "NRORG" => $row['NRORG'],
            "CDLIVROMAG" => $row['CDLIVROMAG'],
            "CDLAYOUTLMAG" => $row['CDLAYOUTLMAG']
        ));
        
    }
    public function deletarCfopTipo($row){
        
        $this->factoryLivroMagnetico->deletaTipoDocumentoCfopTipo(array(
            "NRORG" => $row['NRORG'],
            "CDLIVROMAG" => $row['CDLIVROMAG'],
            "CDTIPOOPER" => $row['CDTIPOOPER'],
            "IDENTRSAIDOP" => $row['IDENTRSAIDOP']
        ));
        
    }
    
    public function generoProd( $row ) {
        $result = $this->factoryLivroMagnetico->getGeneroProd( $this->nrorg, $row['CDGENPROD'], $row['CDLIVROMAG'] );
        
        if( !$result ) {
            $result = $this->factoryLivroMagnetico->getGeneroProd( $this->nrorg, $row['CDGENPROD'], 'MOBILE' );
            $this->factoryLivroMagnetico->insertGeneroProd( $row['CDGENPROD'], $result['DSGENPROD'], $row['CDLIVROMAG'], $this->nrorg, $this->operador );
        }
    }
    
    public function replicaGeneroTipoItemProduto($cdlivromagorig, $cdlivromagdest){
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->factoryLivroMagnetico->replicaGeneroTipoItemProduto(array(
            "NRORG" => $this->nrorg,
            "CDLIVROMAGORIG" => $cdlivromagorig,
            "CDLIVROMAGDEST" => $cdlivromagdest,
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->nrorg,
            "CDOPERINCLUSAO" => $this->operador
        ));
    }

    
}
